﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Windows;
using Telerik.Windows.Documents;
using Telerik.Windows.Documents.Model;

namespace RichTextBoxDemo
{
    /// <summary>
    /// Interaction logic for MainWindow.xaml
    /// </summary>
    public partial class MainWindow : Window
    {
        public MainWindow()
        {
            InitializeComponent();
        }

        private void button1_Click(object sender, RoutedEventArgs e)
        {
            this.AddHeading();
        }

        private void button2_Click(object sender, RoutedEventArgs e)
        {
            this.RemoveHeading();
        }

        private void AddHeading()
        {
            var rtbEditor = new RadDocumentEditor(this.radRichTextBox.Document);

            rtbEditor.BeginUndoGroup();

            var paragraphBox = rtbEditor.Document.CaretPosition.GetCurrentParagraphBox();
            var paragraph = paragraphBox.AssociatedParagraph;
            if (paragraph?.IsEmpty == false)
            {
                rtbEditor.InsertParagraph();
            }

            int headingsCount = 9;
            for (int i = 1; i < headingsCount; i++)
            {
                // preload the built in headings style.
                rtbEditor.Document.StyleRepository.GetValueOrNull(RadDocumentDefaultStyles.GetHeadingStyleNameByIndex(i));

                rtbEditor.Document.Selection.Clear();

                // creating and anchoring the read only start position.
                using (var readOnlyStartPosition = new DocumentPosition(rtbEditor.Document.CaretPosition))
                {
                    readOnlyStartPosition.AnchorToCurrentBoxIndex();

                    rtbEditor.InsertInline(new Span(" "));

                    // creating and anchoring the style to apply start position.
                    using (var styleStartPosition = new DocumentPosition(rtbEditor.Document.CaretPosition))
                    {
                        styleStartPosition.AnchorToCurrentBoxIndex();

                        rtbEditor.Insert("Text " + i);
                        // creating the style to apply end position.
                        using (var styleEndPosition = new DocumentPosition(rtbEditor.Document.CaretPosition))
                        {
                            rtbEditor.Document.Selection.Clear();
                            styleStartPosition.RestorePositionFromBoxIndex();

                            // no need to anchor and restore the style to apply end position as the document is not changed since the creation of this position.
                            rtbEditor.Document.Selection.AddSelectionStart(styleStartPosition);
                            rtbEditor.Document.Selection.AddSelectionEnd(styleEndPosition);
                            rtbEditor.ChangeStyleName(RadDocumentDefaultStyles.GetHeadingCharStyleNameByIndex(i), false);
                            rtbEditor.Document.Selection.Clear();
                        }
                    }

                    rtbEditor.InsertInline(new Span(" "));

                    // creating and anchoring the read only end position.
                    using (var readOnlyEndPosition = new DocumentPosition(rtbEditor.Document.CaretPosition))
                    {
                        rtbEditor.Document.Selection.Clear();
                        readOnlyStartPosition.RestorePositionFromBoxIndex();

                        // no need to anchor and restore the read only range end position as the document is not changed since the creation of this position.

                        rtbEditor.Document.Selection.AddSelectionStart(readOnlyStartPosition);
                        rtbEditor.Document.Selection.AddSelectionEnd(readOnlyEndPosition);
                        rtbEditor.InsertReadOnlyRange();
                        rtbEditor.Document.Selection.Clear();
                    }
                }

                if (i < headingsCount - 1)
                {
                    rtbEditor.InsertParagraph();
                }
            }
            rtbEditor.EndUndoGroup();
        }

        private void RemoveHeading()
        {
            var rtbEditor = new RadDocumentEditor(this.radRichTextBox.Document);
            rtbEditor.RespectReadOnlyRanges = false;

            var spanBox = rtbEditor.Document.CaretPosition.GetCurrentSpanBox();
            if (spanBox == null)
            {
                return;
            }

            Span span = spanBox.AssociatedSpan as Span;
            var sibling = span.PreviousSibling;
            while (sibling != null && !(sibling is ReadOnlyRangeStart) && !(sibling is ReadOnlyRangeEnd))
                sibling = sibling.PreviousSibling;
            var start = sibling as ReadOnlyRangeStart;
            if (start == null)
                return;
            var end = start.End;
            if (end == null)
                return;

            rtbEditor.Document.CaretPosition.MoveToInline(start);

            // creating and anchoring the start position.
            using (var startPosition = new DocumentPosition(rtbEditor.Document.CaretPosition))
            {
                startPosition.AnchorToCurrentBoxIndex();

                rtbEditor.Document.CaretPosition.MoveToInline(end);

                // creating and anchoring the end position.
                using (var endPosition = new DocumentPosition(rtbEditor.Document.CaretPosition))
                {
                    endPosition.AnchorToCurrentBoxIndex();

                    rtbEditor.BeginUndoGroup();
                    rtbEditor.DeleteReadOnlyRange(start);

                    rtbEditor.Document.Selection.Clear();

                    // restoring the positions.
                    startPosition.RestorePositionFromBoxIndex();
                    endPosition.RestorePositionFromBoxIndex();

                    rtbEditor.Document.Selection.AddSelectionStart(startPosition);
                    rtbEditor.Document.Selection.AddSelectionEnd(endPosition);
                    rtbEditor.Delete(false);
                }
            }
            rtbEditor.EndUndoGroup();
        }
    }
}
